#!/usr/bin/python
# # -*- coding:iso-8859-1  -*-

"""
 Name:	      metar2txt.py
 Description: Transform a file with METAR information for one year
              of hourly data 
              in a text file in the ISO 8601 format and with the
              air temperature:
              YYYY-MM-DDTHH:mm, TT

              Input: station_YYYY.txt
              Output: station_YYYY_MM.txt (MM in [1,12])
 
 Notes: 

 Author: Miguel Tremblay
 Date: 18 mai 2006
"""

import sys
import os
import os.path
from datetime import datetime
from datetime import timedelta
import unique

def main(sFichier):

    file_input = open(sFichier, 'r')
    sYear = sFichier.split('_')[1][0:4]
    lMetar = file_input.readlines()

    # Split les entrees par mois

    # Initialisation des dictionnaires
    dMonthMetar = {}
    dMonthTemperature = {}
    for i in range(1,13):
        dMonthMetar[i] = []
        dMonthTemperature[i] = []

    # On separe les METAR par mois.
    for sLine in lMetar:
        nMonth = int(sLine[4:6])
        dMonthMetar[nMonth].append(sLine)

    # On parse le mois par mois et on identifie les elements
    #  de temperature et d'etat.
    for nMonth in range(1,13):
        lMetar = dMonthMetar[nMonth]
        lMetar.sort()        
        sMonth = get_2_nbr(str(nMonth))
        for sMetar in lMetar:
            lMetarElement =sMetar.split()
            if len(lMetarElement) < 4:
                continue
            sDate = lMetarElement[2]
            if len(sDate) > 5: # Format DDHHMMZ
                sDay = sDate[0:2]
                sHour = sDate[2:4]
                sMin = sDate[4:6]
            else: # Format HHMMZ
                sDay = lMetarElement[0][6:8]
                sHour = sDate[0:2]
                sMin = sDate[2:4]
            # Check if date exists for this month.
            try:
                datetime(int(sYear), int(sMonth), int(sDay),\
                         int(sHour), int(sMin))
            except ValueError:
                continue
                
            sISO8601 = sYear + '-' + sMonth + '-' +sDay +\
                       'T' + sHour + ':' + sMin            
            for sMetarElement in lMetarElement[3:]:
                if '/' in sMetarElement[2:] and \
                       '=' not in sMetarElement and \
                       sMetarElement[0] != '/' and \
                       '-' not in sMetarElement and \
                       '/' not in sMetarElement[-1] and \
                       'SE' not in sMetarElement and \
                       'R' not in sMetarElement and \
                       'L' not in sMetarElement and \
                       'V' not in sMetarElement and \
                       'H' not in sMetarElement and \
                       'F' not in sMetarElement and \
                       '<' not in sMetarElement and \
                       'SM' not in sMetarElement and \
                       'KT' not in sMetarElement:

                    sTemperature = sMetarElement.split('/')[0]
                    nTemperature = int(round(float\
                                             (sTemperature.replace('M','-'))))
                    (b_pluie, sEtat) = y_mouille_tu(sMetar)
                    sLine = sISO8601 + ',' + str(nTemperature) +\
                            ',' + sEtat + ',' + str(b_pluie)
                    dMonthTemperature[nMonth].append(sLine)
                    break

    write_data(dMonthTemperature, sFichier)

def get_2_nbr(sString):
    """
    Si une string a une longueur de 1, on ajoute un 0 au debut.
    Particulierement utile pour les cas de date.
    """
    if len(sString) == 1:
        sString = '0' + sString

    return sString

def write_data(dMonthTemperature, sPathInput):
    """
    Ecrire les fichiers textes dans un repertoire nomme 'txt'.
    Le dictionnaire en entree contient les donnes pour une annee entiere,
    il faut creer un fichier par mois et ecrit de la facon suivante:
    station_YYYY_MM.txt

    sFichierInput est utilise pour connaitre le repertoire racine.
    """
    (sDirectoryInput, sFichierInput) = os.path.split(sPathInput)
    sDirectoryOutput = sDirectoryInput + '/' + 'txt'
    if not os.path.exists(sDirectoryOutput):
        print 'Creating:',sDirectoryOutput
        os.mkdir(sDirectoryOutput)
    
    for nMonth in dMonthTemperature.keys():
        if len(dMonthTemperature[nMonth]) == 0:
            continue
        lOutput = add_time(dMonthTemperature[nMonth])
        sMonth = dMonthTemperature[nMonth][0][5:7]
        sFilenameOutput = sFichierInput[:-4] + '_' + sMonth + '.txt'
        sOutput = os.path.join(sDirectoryOutput, sFilenameOutput)
        print "Writing:", sOutput
        fichier_output = open(sOutput, 'w')
        fichier_output.writelines(lOutput)

def add_time(lListe):
    """
    Calcule le l'aire en dessous de la courbe de la temperature en fonction
    du temps. On utilise la methode des trapezes pour interpoler la fonction.
    Voir http://fr.wikipedia.org/wiki/M%C3%A9thode_des_trap%C3%A8zes

    Input: Liste de string contenant:
              'ISO8601, temperature, etat, pluie [binaire]'


    Output: Liste ordonne avec 
         ISO8601, temperature, etat, pluie [binaire], aire
    """

    # On commence par enlever les doublons
    lListeUnique = unique.unique(lListe)
    # On met les affaires en ordre.
    lListeUnique.sort()

    # On cree une objet datetime avec chacune des dates
    lDateTime = []
    lTemperature = []
    for sLine in lListeUnique:
        lLine = sLine.split(',')
        sDateISO8601 = lLine[0]
        nTemperature = int(lLine[1])
        nYear = int(sDateISO8601[0:4])
        nMonth = int(sDateISO8601[5:7])
        nDay = int(sDateISO8601[8:10])
        nHour = int(sDateISO8601[11:13])
        nMinute = int(sDateISO8601[14:16])
        lDateTime.append(datetime(nYear, nMonth, nDay, nHour, nMinute))
        lTemperature.append(nTemperature)

    # On calcule l'aire pour chaque point. On met a chaque ligne l'aire
    #  qu'il y a entre ce point et le suivant. L'aire du dernier point est 0.
    lListeUnique[-1] = lListeUnique[-1] + ',0'
    for i in range(0, len(lDateTime)-1):
        fHour = (lDateTime[i]-lDateTime[i-1]).seconds/3600.0
        fArea = fHour * (lTemperature[i]+lTemperature[i+1])/2
        lListeUnique[i] = lListeUnique[i] + ',' + str(fArea) + '\n'

    return lListeUnique

    
def y_mouille_tu(sMetar):
    """
    Regarder si l'etat peut etre considerer comme etant de la precipitation.
    La liste des etats peut etre trouve ici:
    http://www.nws.noaa.gov/oso/oso1/oso12/fmh1/fmh1ch8.htm#chp8link
    
    Note: Les metar etant en uppercase, on va travailler comme ca.

    Input: sMetar qui est la ligne metar au complet
    Output: (b_mouille, sEtat)
          b_mouille == 1 si l'etat est comme mouille,
          b_mouille == 0 sinon.
          sEtat est l'etat trouve, sinon sa valeur est 'sec'
    """
    lEtatMouille = ['DZ', 'RA', 'SN', 'SG', 'IC', 'PE', 'GR', 'GS' 'UP']

    for sEtatMouille in lEtatMouille:
        if sEtatMouille in sMetar:
            return (1, sEtatMouille)
    return (0, 'sec')


if __name__ == "__main__":

    lArgs = ['']*1

    if len(sys.argv) < 2:
        print "Usage metar2txt.py fichier_station.txt"
        sys.exit(1)
        
    sFichier = sys.argv[1]

    
    main(sFichier)

